/**
  ********************************(C) COPYRIGHT 2017 DJI************************
  * @file       gimbal_task.c
	* @brief      basic gimbal control and autoshoot task
	* @update	        
	*	@history
	* Version     Date          Author           Modification
  * V1.0.0      Apr-30-2017   Richard.luo 
  * @verbatim
	* yaw axis  : angle feedback is single axis gyro, palstance feedback is mpu6500 gyro
	* pitch axis: angle feedback is encoder, palstance feedback is mpu6500 gyro
	********************************(C) COPYRIGHT 2017 DJI************************
	*/

#include "gimbal_task.h"
#include "bsp_can.h"
#include "bsp_uart.h"
#include "calibrate.h"
#include "chassis_task.h"
#include "cmsis_os.h"
#include "error_task.h"
#include "judge_sys.h"
#include "kb.h"
#include "mpu.h"
#include "mytype.h"
#include "pid.h"
#include "stm32f4xx_hal.h"
#include "sys.h"
#include <math.h>


/* for debuge */
int pit_p_r, pit_p_f, pit_s_r, pit_s_f;
int yaw_p_r, yaw_p_f, yaw_s_r, yaw_s_f;

gimbal_yaw_t gYaw;

/* gimbal pid parameter */
float yaw_angle_ref     = 0;
float pit_angle_ref     = 0; 
float yaw_angle_fdb     = 0;
float pit_angle_fdb     = 0; 
float yaw_speed_ref     = 0;
float pit_speed_ref     = 0;
float yaw_speed_fdb     = 0;
float pit_speed_fdb     = 0;
float pit_kp = 0;
float yaw_kp = 0;

float yaw_speed_ref_offset = 0;


/* read from flash */
int   pit_center_offset = 100; 
int   yaw_center_offset = 0;

float	armor_width = 0.075;

/*define users offset*/
#define YAW_CALIB_OFFSET 250
#define PITCH_CALIB_OFFSET -68

/* gimbal relative position param */
s16   pit_relative_pos;
s16   yaw_relative_pos;   //unit : encoder
float pit_relative_angle;
float yaw_relative_angle; //unit : degree
		
/* shoot task relevant param */
int   last_sw1;
int   auto_shoot_cmd = 0;
int 	need_move_cmd = 0;

int 	aim_ok_flag = 0;
int   fric_wheel_run = 0;
int   trigger_spd_ref, trigger_pos_ref; 
int   trigger_dir = 1;

/* imu relevant param */
float imu_tmp;
float imu_tmp_ref = 40; //keep imu in 40 degree


/* whether chassis follow gimbal when autoshoot */
int auto_chassis_follow_gim = 1;

/* automatic aim param */
int AimDirectionBiasYaw = -0;
int AimDirectionBiasPit = 0;
int CompensationY = 65;

/* vision data */
int YawBias = 0;
int PitBias = 0;
float PitchTarget = 0; 
float YawTarget = 0;
float LastPitchTarget = 0; 
float LastYawTarget = 0;
float RatioPitch = 40;
float RatioYaw = 10;
uint32_t no_vision_time;
double pnp_dis;


/******search time**********/
float search_begin_time;

/***********camera Martix*********/
float M1[3][4] = {{1223.464 ,		 	 0,	293.504,	0},
									{				0,1228.241,	285.957,	0},
									{				0,			 0,			  1,	0}};

float M2[4][4] = {{ 1,	0,	0,	0},
									{	0,	1,	0,	0.045},
									{	0,	0,	1,	0},
									{	0,	0,	1,	0}};

/*************KLM fliter***********/
KLM_Fliter visionX_klm_fliter;
KLM_Fliter visionY_klm_fliter;
KLM_Fliter visionD_klm_fliter;
/**
  * @brief     get relative position angle to center
  * @param[in] raw_ecd: gimbal motor encoder raw angle
	* @param[in] center_offset: read GimbalCaliData from chip flash
	* @retval    relative angle, unit is degree.
  * @attention you should read center offset data in chip flash, 
	*            as: GimbalCaliData.GimbalPit/Yaw Offset
  */
s16 get_relative_pos(s16 raw_ecd, s16 center_offset)
{
    s16 tmp = 0;
    if (center_offset >= 4096)
    {
        if (raw_ecd > center_offset - 4096)
            tmp = raw_ecd - center_offset;
        else
            tmp = raw_ecd + 8192 - center_offset;
    }
    else
    {
        if (raw_ecd > center_offset + 4096)
            tmp = raw_ecd - 8192 - center_offset;
        else
            tmp = raw_ecd - center_offset;
    }
    return tmp;
}
/**
  * @brief     send current which pid calculate to esc. message to calibrate 6025 gimbal motor esc
  * @param     current value corresponding motor(yaw/pitch/trigger)
  */
void can_send_gimbal_iq(s16 yaw_iq, s16 pit_iq, s16 trigger_iq)
{

    GIMBAL_CAN.pTxMsg->StdId   = 0x1ff;
    GIMBAL_CAN.pTxMsg->IDE     = CAN_ID_STD;
    GIMBAL_CAN.pTxMsg->RTR     = CAN_RTR_DATA;
    GIMBAL_CAN.pTxMsg->DLC     = 8;
    GIMBAL_CAN.pTxMsg->Data[0] = yaw_iq >> 8;
    GIMBAL_CAN.pTxMsg->Data[1] = yaw_iq;
    GIMBAL_CAN.pTxMsg->Data[2] = pit_iq >> 8;
    GIMBAL_CAN.pTxMsg->Data[3] = pit_iq;
    GIMBAL_CAN.pTxMsg->Data[4] = trigger_iq >> 8;
    GIMBAL_CAN.pTxMsg->Data[5] = trigger_iq;
    GIMBAL_CAN.pTxMsg->Data[6] = 0;
    GIMBAL_CAN.pTxMsg->Data[7] = 0;
    HAL_CAN_Transmit(&GIMBAL_CAN, 1000);
}
/**
  * @brief     send particular message to calibrate 6025/6623 gimbal motor esc
  * @usage     only need calibrate motor when replace a new ESC
  */
void can_cali_6025()
{

    GIMBAL_CAN.pTxMsg->StdId   = 0x3f0;
    GIMBAL_CAN.pTxMsg->IDE     = CAN_ID_STD;
    GIMBAL_CAN.pTxMsg->RTR     = CAN_RTR_DATA;
    GIMBAL_CAN.pTxMsg->DLC     = 8;
    GIMBAL_CAN.pTxMsg->Data[0] = 'c';
    GIMBAL_CAN.pTxMsg->Data[1] = 0;
    GIMBAL_CAN.pTxMsg->Data[2] = 0 >> 8;
    GIMBAL_CAN.pTxMsg->Data[3] = 0;
    GIMBAL_CAN.pTxMsg->Data[4] = 0 >> 8;
    GIMBAL_CAN.pTxMsg->Data[5] = 0;
    GIMBAL_CAN.pTxMsg->Data[6] = 0;
    GIMBAL_CAN.pTxMsg->Data[7] = 0;
    HAL_CAN_Transmit(&GIMBAL_CAN, 1000);
}
void can_cali_6623()
{

    GIMBAL_CAN.pTxMsg->StdId   = 0x1ff;
    GIMBAL_CAN.pTxMsg->IDE     = CAN_ID_STD;
    GIMBAL_CAN.pTxMsg->RTR     = CAN_RTR_DATA;
    GIMBAL_CAN.pTxMsg->DLC     = 8;
    GIMBAL_CAN.pTxMsg->Data[0] = 0;
    GIMBAL_CAN.pTxMsg->Data[1] = 0;
    GIMBAL_CAN.pTxMsg->Data[2] = 0 >> 8;
    GIMBAL_CAN.pTxMsg->Data[3] = 0;
    GIMBAL_CAN.pTxMsg->Data[4] = 0 >> 8;
    GIMBAL_CAN.pTxMsg->Data[5] = 0;
    GIMBAL_CAN.pTxMsg->Data[6] = 4;
    GIMBAL_CAN.pTxMsg->Data[7] = 0;
    HAL_CAN_Transmit(&GIMBAL_CAN, 1000);
}

void auto_search_init(void)
{
	search_begin_time = HAL_GetTick() * 0.001;
}

void auto_search(void)
{
	//float one_area_time = 1.0;
	float max_search_angle = 40.0;
	float min_search_angle = -40.0;
	static int8_t search_dir = 1;
	float search_range = max_search_angle - min_search_angle;
	if (ReceData.visionDataStatus == 0)
	{

		if(chassis.auto_mode == CHASSIS_AUTO_SEARCH_ATTACKER)
		{
			switch (testRealBloodChangedData.weakId)
			{
				case 1:
					search_dir = -1;
					break;
				case 2:
					search_dir = -1;
					break;
				case 3:
					search_dir = 	1;
					break;
			}
			YawTarget = yaw_relative_angle + search_dir * 2;
		}
		else
		{
			YawTarget = yaw_relative_angle + search_dir * 2;
		}
		
		if(YawTarget > max_search_angle)
		{
			search_dir = -1;
		}
		else if(YawTarget < min_search_angle)
		{
			search_dir = 1;
		}
		PitchTarget = -12.0f;
		
		
	}
	else if(ReceData.visionDataStatus == 1)
	{
		gYaw.last_mode = gYaw.ctrl_mode;
		gYaw.ctrl_mode = GIMBAL_AUTO_SHOOT;
		chassis.auto_shoot_mode = CHASSIS_AUTO_SHOOT_STAND;
		
		pit_angle_fdb = pit_relative_angle;
		yaw_angle_fdb = yaw_zgyro_angle - gYaw.zgyro_offset;
		pit_angle_ref = pit_angle_fdb;
		yaw_angle_ref = yaw_angle_fdb;
		YawTarget = yaw_angle_fdb;
		PitchTarget = pit_angle_fdb;
	}
}






void auto_shoot_pid_init(void)
{
		PID_struct_init(&pid_pit, POSITION_PID, 6000, 1000,
									300.0f, 0.3, 1); //
	PID_struct_init(&pid_pit_speed, POSITION_PID, 20000, 2000,
									30.0f, 0.3, 3.0f);
	
	PID_struct_init(&pid_yaw, POSITION_PID, 7000, 1000,
									300, 0.3, 2.0); //
	PID_struct_init(&pid_yaw_speed, POSITION_PID, 20000, 2000,
									30, 0.3, 5);
}
void normal_pid_init(void)
{
	                                       //maxout   intergral_limit	
	PID_struct_init(&pid_pit, POSITION_PID, 8000, 1000,
									200.0f, 0.2, 4); //
	PID_struct_init(&pid_pit_speed, POSITION_PID, 10000, 800,
									30.0f, 0.3, 3.0f);

	PID_struct_init(&pid_yaw, POSITION_PID, 4000, 300,
									100, 2.0, 10.0); //
	PID_struct_init(&pid_yaw_speed, POSITION_PID, 10000, 800,
									30, 0.3, 3);
}
void gimbal_pid_init(void)
{
  normal_pid_init();	

	PID_struct_init(&pid_trigger, POSITION_PID, 10000, 2000, //position
									15.0f, 0, 10);
	PID_struct_init(&pid_trigger_speed, POSITION_PID, 10000, 4000, //vw
									1.5f, 0.3f, 5);

	PID_struct_init(&pid_imu_tmp, POSITION_PID, 999, 999, //position
									180, 0.1f, 0);
}



void KLM_Fliter_struct_init(KLM_Fliter* klm_fliter,float Q,float R)
{
	klm_fliter->Q = Q;
	klm_fliter->R = R;
}

void fliters_init(void)
{
	KLM_Fliter_struct_init(&visionX_klm_fliter,0,0);
	KLM_Fliter_struct_init(&visionY_klm_fliter,0,0);
	KLM_Fliter_struct_init(&visionD_klm_fliter,0,0);
}

void KLM_Fliter_begin(KLM_Fliter* klm_fliter,float x_measure)
{
	klm_fliter->Zmeasure = x_measure;
	klm_fliter->x_last = x_measure;
	klm_fliter->x_mid = x_measure;
	klm_fliter->x_now = x_measure;
	klm_fliter->v_diff = 0;
}

float KLM_Fliter_calc(KLM_Fliter* klm_fliter, float x_measure)
{
	float kg = 0;

	klm_fliter->x_mid = klm_fliter->x_last; //+ klm_fliter->v_diff;
	klm_fliter->p_mid = klm_fliter->p_last + klm_fliter->Q;
	kg = klm_fliter->p_mid/(klm_fliter->p_mid + klm_fliter->R);
	klm_fliter->x_now = klm_fliter->x_mid + kg * (x_measure - klm_fliter->x_mid);
	klm_fliter->p_now = (1 - kg) * klm_fliter->p_mid;
	klm_fliter->v_diff = klm_fliter->x_now - klm_fliter->x_last;
	klm_fliter->p_last = klm_fliter->p_now;  
  klm_fliter->x_last = klm_fliter->x_now; 
	
	return klm_fliter->x_now;
	
}


void shoot_task()
{
	static int kadan_count = 0;
	static int kadan_count_limition = 50;
	static float kadan_begin_time = 0.0f;
	static uint8_t kadan_need_init_flag = 0;
	static float kadan_angle;
	

	
	if(gYaw.trigger_mode == CONTINUE_MODE && auto_shoot_cmd == 1 && abs(moto_trigger.speed_rpm) < 10)
	{
		kadan_count ++;
	}
	else
	{
		kadan_count = 0;
	}
	if(gYaw.trigger_mode == CONTINUE_MODE && kadan_count > kadan_count_limition)
	{
		gYaw.trigger_mode = KA_DAN_MODE;
		kadan_begin_time = HAL_GetTick() * 0.001;
		kadan_need_init_flag = 1;
		kadan_count = 0;
	}
	else if (gYaw.trigger_mode == KA_DAN_MODE && (HAL_GetTick()* 0.001 - kadan_begin_time) > 0.3f)
	{
		gYaw.trigger_mode = CONTINUE_MODE;
	}
	
	
	
	switch(gYaw.trigger_mode)
	{
		
		case CONTINUE_MODE:		
		if (fric_wheel_run && auto_shoot_cmd)
		{
			trigger_spd_ref = -15000;
			chassis.judgeInfo.trigger_state = 1;
		}
		else
		{
			trigger_spd_ref = 0;
			chassis.judgeInfo.trigger_state = 0;
		}
	break;
		case KA_DAN_MODE:	
			if(kadan_need_init_flag)
			{
				kadan_angle = moto_trigger.total_ecd;
				kadan_need_init_flag = 0;
			}
			pid_calc(&pid_trigger, moto_trigger.total_ecd / 100, (kadan_angle + 130922) / 100);
			trigger_spd_ref = pid_trigger.pos_out;
			break;
}
		//trigger_pos_ref += trigger_MOTOR_SPEED * trigger_dir;

		

	if (gYaw.ctrl_mode == GIMBAL_AUTO_SHOOT ||  gYaw.ctrl_mode == GIMBAL_SEARCH_TARGET)//&& chassis.mode == CHASSIS_AUTO)
	{
		TIM12->CCR1 = TIM12->CCR2 = SHOT_FRIC_WHEEL_SPEED; 
		HAL_GPIO_WritePin(GPIOG, GPIO_PIN_13, GPIO_PIN_SET);		
		fric_wheel_run = 1;
		//fric_wheel_run = 0;
	}
	else
	{
		TIM12->CCR1 = TIM12->CCR2 = 1000;
		HAL_GPIO_WritePin(GPIOG, GPIO_PIN_13, GPIO_PIN_RESET);	
		fric_wheel_run = 0;
	}
	last_sw1 = rc.sw1;

}
#define MY_ALIG 1
#if MY_ALIG
void auto_aim_shoot(void)
{
//	static uint8_t get_vision_data_flag = 0;
	if (ReceData.visionDataStatus == 1)
	{
		
		/*
				if (VisionDataX < 5 && VisionDataX > -5)
		{
			VisionDataX = 0;
		}
		*/
		//calculate pitch elevation
		//you should modify these parameter
		/*640 480
		float YawDataK = 0.5;
		float PitchDataK = 0.3;
		*/
		float YawDataK = 0.3;
		float PitchDataK = 0.3;
		float YawSpeedPredictK = 0.0;
		float PitchSpeedPredictK = 0.12;
		
		float PitchError = 0.0;
		float YawError = 0.0;
		
//		static float last_visionD = 0;
	//	static float last_visionX = 0;
	//	static float last_visionY = 0;
		
		
		pnp_dis = VisionDataDis * 0.001 + 0.2;
		
	
		PitchError 	= (57 *(-VisionDataY )/FY+ PitchSpeedPredictK * pnp_dis) + 3.5f;
		YawError 		= 57 *	 (-VisionDataX )/FX + YawSpeedPredictK * imu.wz * pnp_dis +0.75f ;//- yaw_speed_ref_offset * 3.0;
		VAL_LIMIT(YawError, -10,10);
		
		
		PitchTarget =  pit_angle_fdb +  PitchDataK * PitchError;// 57.2 = 180/3.14
		YawTarget = yaw_angle_fdb 	+ 	YawDataK * YawError;
			

		if(((PitchError > -2.0f) && (PitchError < 2.0f)) 
			&& (abs(YawError) < armor_width / pnp_dis * 57)
			&&	abs((imu.wz)*57) < 10)// && (YawError > - armor_width / pnp_dis * 57) && (YawError < armor_width / pnp_dis * 57))// && (VisionDataDis <= 5000))
		{
			auto_shoot_cmd = 1;
			
		}
		else
		{
			auto_shoot_cmd = 0;
		}		
		if(((PitchError > -2.0f) && (PitchError < 2.0f)) 
			&& (abs(YawError) < armor_width / pnp_dis * 57/2)
			&&	abs((imu.wz)*57) < 10)
				{
					aim_ok_flag = 1;
				}
				else
				{
					aim_ok_flag = 0;
				}
		
		
		
		

		
		//VAL_LIMIT(PitchTarget, pit_angle_fdb - 20.0f, pit_angle_fdb + 20.0f);								
		VAL_LIMIT(PitchTarget, -20,30);
		LastPitchTarget = PitchTarget;
		
		//VAL_LIMIT(YawTarget, yaw_angle_fdb - 20.0f, yaw_angle_fdb + 20.0f);

		LastYawTarget = YawTarget;
		

		
		YawBias = YawTarget - yaw_angle_fdb;
		
		
	}

	else if (ReceData.visionDataStatus == 2)
	{
		//
		PitchTarget = VisionDataY * 57.3 / 1000.0;
		YawTarget   = VisionDataX * 57.3 / 1000.0;
		
		if((yaw_relative_angle - YawTarget > -2) 
			  && (yaw_relative_angle - YawTarget < 2))
		{
			auto_shoot_cmd = 1;
		}
		else
		{
			auto_shoot_cmd = 0;
		}
	}
	else
	{
		auto_shoot_cmd = 0;
		
		if (last_vision_status == 1)
		{
			no_vision_time = HAL_GetTick();
		}
		
		//if chassis follow gimbal when auto shooting lost vision
		//yaw return center immediately 
		/*
		if (auto_chassis_follow_gim == 1)
		{
			YawTarget = 0;
		}
		*/
		if (HAL_GetTick() - no_vision_time > 1000)
		{
			//YawTarget = 0;
			PitchTarget = 0;
			//get_vision_data_flag = 0;
			gYaw.last_mode = gYaw.ctrl_mode;
			gYaw.ctrl_mode = GIMBAL_SEARCH_TARGET;
			
			pit_angle_fdb = pit_relative_angle;
			yaw_angle_fdb = yaw_relative_angle;
			pit_angle_ref = pit_angle_fdb;
			yaw_angle_ref = yaw_angle_fdb;
		}
	}
	
}

#else
void auto_aim_shoot(void)
{
	if (ReceData.visionDataStatus == 1)
	{
		/*
				if (VisionDataX < 5 && VisionDataX > -5)
		{
			VisionDataX = 0;
		}
		*/
		//calculate pitch elevation
		//you should modify these parameter
		//according your fitting data
		float yaw_predict_K = 0.0;
		pnp_dis = VisionDataDis * 0.001;
		AimDirectionBiasPit = (int)(21.7617
		                      - 36.7245 * pnp_dis
		                      + 5.8522 * pnp_dis * pnp_dis
		                      - 0.4506 * pnp_dis * pnp_dis * pnp_dis);
		
		AimDirectionBiasYaw = yaw_predict_K * imu.wz * pnp_dis;
		
		YawBias = - (VisionDataX + AimDirectionBiasYaw);
		PitBias = - (VisionDataY + AimDirectionBiasPit);
		
		
		PitchTarget =  pit_relative_angle + PitBias/RatioPitch;
		VAL_LIMIT(PitchTarget, -15, 17);
		
		LastPitchTarget = PitchTarget;
		
		YawTarget =  yaw_relative_angle + YawBias/RatioYaw ;
		VAL_LIMIT(YawTarget, -40, 40);
		LastYawTarget = YawTarget;
		
		if((YawBias > -15) && (YawBias < 15))// && (VisionDataDis <= 5000))
		{
			auto_shoot_cmd = 1;
		}
		else
		{
			auto_shoot_cmd = 0;
		}
	}

	else if (ReceData.visionDataStatus == 2)
	{
		//
		PitchTarget = VisionDataY * 57.3 / 1000.0;
		YawTarget   = VisionDataX * 57.3 / 1000.0;
		
		if((yaw_relative_angle - YawTarget > -2) 
			  && (yaw_relative_angle - YawTarget < 2))
		{
			auto_shoot_cmd = 1;
		}
		else
		{
			auto_shoot_cmd = 0;
		}
	}
	else
	{
		auto_shoot_cmd = 0;
		
		if (last_vision_status == 1)
		{
			no_vision_time = HAL_GetTick();
		}
		
		//if chassis follow gimbal when auto shooting lost vision
		//yaw return center immediately 
		if (auto_chassis_follow_gim == 1)
		{
			YawTarget = 0;
		}
		
		if (HAL_GetTick() - no_vision_time > 2000)
		{
			YawTarget = 0;
			PitchTarget = 0;
		}
	}
	
}
#endif
void getCaliData(void)
{
	
	if (gAppParam.GimbalCaliData.isAlreadyCalied == CALIED_FLAG)
	{
		yaw_center_offset = gAppParam.GimbalCaliData.GimbalYawOffset + YAW_CALIB_OFFSET;
		pit_center_offset = gAppParam.GimbalCaliData.GimbalPitOffset + PITCH_CALIB_OFFSET;
	}
	else
	{
		//while (1);
	}
}
/**
  * @brief     gimbal control task
  * @attention the yaw angle feedback is single axis gyro, palstance feedback is mpu6500 gyro
  *            so maybe happen positive feedback, 
  *            if this case, you should modify their parameter sign.
  */
void gimbal_task(const void* argu)
{
	gimbal_pid_init();

	//mpu6500 & ist3810 magnet meter init ,for angular rate measure
	mpu_device_init();
	//read gimbal offset from gAppParam
	getCaliData();
//		can_cali_6025();
//		can_cali_6623();
	yaw_relative_pos = -get_relative_pos(moto_yaw.angle, yaw_center_offset);
	pit_relative_pos = get_relative_pos(moto_pit.angle, pit_center_offset);			
		
	pit_relative_angle = pit_relative_pos/22.75f;
	yaw_relative_angle = yaw_relative_pos/22.75f;	
	
	gYaw.zgyro_offset = chassis.angle_from_gyro - yaw_relative_angle;

	while (1)
	{
		//keep imu in a constant temperature
		imu_tmp = 21 + mpu_data.temp / 333.87f;
		pid_calc(&pid_imu_tmp, imu_tmp, imu_tmp_ref); //keep in 40 degree
		TIM3->CCR2 = pid_imu_tmp.pos_out;

		// get Z axis palstance, for gimbal speed loop
		mpu_get_data();
		gimbalCaliHook();

		if (gYaw.last_mode == GIMBAL_RELAX && gYaw.ctrl_mode != GIMBAL_RELAX)
			pit_angle_ref = 0; //back to center
		gYaw.last_mode = gYaw.ctrl_mode;

		switch (rc.sw2)
		{
			case (RC_UP):
				gYaw.last_mode = gYaw.ctrl_mode;
				gYaw.ctrl_mode = GIMBAL_CLOSE_LOOP_ZGYRO;
				//gYaw.ctrl_mode = GIMBAL_CLOSE_LOOP_ENCODER;
			break;
			case (RC_MI):
				gYaw.last_mode = gYaw.ctrl_mode;
				gYaw.ctrl_mode = GIMBAL_RELAX;
			break;
			case (RC_DN):
				if(gYaw.ctrl_mode != GIMBAL_AUTO_SHOOT && gYaw.ctrl_mode != GIMBAL_SEARCH_TARGET)
				{
					gYaw.last_mode = gYaw.ctrl_mode;
					gYaw.ctrl_mode = GIMBAL_SEARCH_TARGET;
				}
				else
				{
					
				}
			break;
			default:
			break;
		}

		//shoot trigger moto task
		shoot_task();
		//transform absolute ecd range [0,8192] to relative range [-MOVE_RANGE, MOVE_RANGE],
		yaw_relative_pos = -get_relative_pos(moto_yaw.angle, yaw_center_offset);
		pit_relative_pos = get_relative_pos(moto_pit.angle, pit_center_offset);			
		
		pit_relative_angle = pit_relative_pos/22.75f;
		yaw_relative_angle = yaw_relative_pos/22.75f;	
	
		//gYaw.ctrl_mode = GIMBAL_AUTO_SHOOT;
		switch (gYaw.ctrl_mode)
		{
			case GIMBAL_AUTO_SHOOT:
			{
				auto_shoot_pid_init();
				if (gYaw.last_mode != GIMBAL_AUTO_SHOOT)
				{
					auto_shoot_pid_init();
				}
				pit_angle_fdb = pit_relative_angle;
				//yaw_angle_fdb = yaw_relative_angle;
				yaw_angle_fdb = yaw_zgyro_angle - gYaw.zgyro_offset;
				
				if (1)//chassis.mode == CHASSIS_AUTO)
				{	
					auto_aim_shoot();
					if((chassis.auto_shoot_mode == CHASSIS_AUTO_SHOOT_STAND
						|| chassis.auto_shoot_mode == CHASSIS_AUTO_SHOOT_STAND_LEFT
						|| chassis.auto_shoot_mode == CHASSIS_AUTO_SHOOT_STAND_RIGHT))
						//&& abs(chassis.vw) <40)
					{
						pit_angle_ref = PitchTarget;
						yaw_angle_ref = YawTarget;
					}
					else if(chassis.auto_shoot_mode == CHASSIS_AUTO_SHOOT_ROTIATION)
					{
						pit_angle_ref = PitchTarget;
						yaw_angle_ref = YawTarget;
					}
					else if (chassis.auto_shoot_mode == CHASSIS_AUTO_SHOOT_TRANSVERSE)
					{
						pit_angle_ref = pit_angle_ref;
						yaw_angle_ref = YawTarget + transerve_glimbal_yaw_offset * 57.0f;
					}
						//pit_angle_ref = PitchTarget;
						//yaw_angle_ref = YawTarget;
					
				}
					
				else
				{
					
					pit_angle_ref = 0;
					yaw_angle_ref = 0;
				}
			
			}break;
			case GIMBAL_SEARCH_TARGET:
				
				
				if (gYaw.last_mode != GIMBAL_SEARCH_TARGET)
				{
					auto_search_init();
				}
				auto_shoot_cmd = 0;
				pit_angle_fdb = pit_relative_angle;
				yaw_angle_fdb = yaw_relative_angle;
				if (1)//chassis.mode == CHASSIS_AUTO)
				{	
					auto_search();
					pit_angle_ref = 0;
					yaw_angle_ref = YawTarget;
				}
				break;
			case GIMBAL_CLOSE_LOOP_ZGYRO:
			{
				if (gYaw.last_mode != GIMBAL_CLOSE_LOOP_ZGYRO)
				{
					normal_pid_init();
			
				}
				
				
				pit_angle_fdb = pit_relative_angle;
				pit_angle_ref += rc.ch4 * 0.003f * GIMBAL_RC_MOVE_RATIO_PIT;
				
#define GY_YAW_USED 1
				
				#if GY_YAW_USED
						yaw_angle_fdb = yaw_zgyro_angle - gYaw.zgyro_offset;
					#else
						yaw_angle_fdb = yaw_relative_angle;
					#endif
					yaw_angle_ref += -rc.ch3 * 0.0015f * GIMBAL_RC_MOVE_RATIO_YAW;
				
				if (chassis.mode == CHASSIS_FOLLOW_GIMBAL_ENCODER)
				{
					#if GY_YAW_USED
						yaw_angle_fdb = yaw_zgyro_angle - gYaw.zgyro_offset;
					#else
						yaw_angle_fdb = yaw_relative_angle;
					#endif
					yaw_angle_ref += -rc.ch3 * 0.0015f * GIMBAL_RC_MOVE_RATIO_YAW;
				}
				
				else
				{
					yaw_angle_fdb = yaw_relative_angle;
					yaw_angle_ref += -rc.ch3 * 0.0015f * GIMBAL_RC_MOVE_RATIO_YAW;
				}
			}break;
			
			default:
			{
			}break;
		}

		
		
		VAL_LIMIT(pit_angle_ref, -10, 25);
		pit_speed_fdb = mpu_data.gx / 10.0f;
		pid_calc(&pid_pit, pit_angle_fdb, pit_angle_ref);
		if (gYaw.ctrl_mode == GIMBAL_AUTO_SHOOT)
			pit_speed_ref = (pid_pit.pos_out)/10.0f;// - pit_kp*mpu_data.gx) / 10.0f;
		else
			pit_speed_ref = pid_pit.pos_out / 10.0f;
		//if(!auto_shoot_cmd)
		//{
		//	pit_speed_ref = 2 * pid_pit.pos_out / 10.0f;
		//}
		//pit_speed_ref = 10.0;
		pid_calc(&pid_pit_speed, pit_speed_fdb, pit_speed_ref);
		//pid_pit_speed.pos_out = 0.0;
		
		
		//VAL_LIMIT(yaw_angle_ref, -45, 45);
		yaw_speed_fdb = mpu_data.gz / 10.0f;
		if(gYaw.ctrl_mode == GIMBAL_AUTO_SHOOT && VisionDataDis > 0)
		{
			yaw_speed_ref_offset = v_ys *sin(yaw_relative_angle / 57) * - v_xs * cos(yaw_relative_angle / 57) / VisionDataDis ;
		}
		pid_calc(&pid_yaw, yaw_angle_fdb, yaw_angle_ref );
		if (gYaw.ctrl_mode == GIMBAL_AUTO_SHOOT)
			yaw_speed_ref = (pid_yaw.pos_out + 0.0 * w_v)/10.0f;// - yaw_kp*mpu_data.gz) / 10.0f;
		else
			yaw_speed_ref = pid_yaw.pos_out / 10.0f;
		//if(!auto_shoot_cmd)
		//{
			//yaw_speed_ref = 1.5 * pid_yaw.pos_out / 10.0f;
		//}
		//yaw_speed_ref = 10.0f;
		pid_calc(&pid_yaw_speed, yaw_speed_fdb, yaw_speed_ref);
	//pid_yaw_speed.pos_out = 500.0f;

		//pid_calc(&pid_trigger, moto_trigger.total_ecd / 100, trigger_pos_ref / 100);
		//pid_calc(&pid_trigger_speed, moto_trigger.speed_rpm, pid_trigger.pos_out);
		pid_calc(&pid_trigger_speed, moto_trigger.speed_rpm, trigger_spd_ref);
		
		//final output, for safe protect purpose
		if (rc.sw2 != RC_MI && gAppParam.GimbalCaliData.isAlreadyCalied == 0x55
				&& gYaw.ctrl_mode != GIMBAL_RELAX
				&& !gRxErr.err_list[DbusTOE].err_exist
				&& !gRxErr.err_list[GimbalYawTOE].err_exist
				&& !gRxErr.err_list[GimbalPitTOE].err_exist)
		{
			can_send_gimbal_iq(pid_yaw_speed.pos_out + yaw_speed_ref_offset* 000.0f, -(pid_pit_speed.pos_out + 0), pid_trigger_speed.pos_out);
		}
		else
		{
			pid_trigger.iout = 0;
			can_send_gimbal_iq(0, 0, 0); //relax state
		}
		
	  /* for debuge */
		pit_p_f = pit_angle_fdb * 1000;
		pit_p_r = pit_angle_ref * 1000;
		yaw_p_f = yaw_angle_fdb * 1000;
		yaw_p_r = yaw_angle_ref * 1000;

		osDelay(5);
	}
}
